package app.mar.utils;

import java.io.Serializable;
import java.util.ArrayList;

import app.mar.utils.game.Resource;

/**
 * Created by ptitcois on 18/08/16.
 * Cette classe gère les differentes ressources
 */
public class ResourceManager implements Serializable{
    protected ArrayList<Resource> mAllResources = new ArrayList<Resource>();
    protected ArrayList< ArrayList<Resource> > mResourcesAquiered  = new ArrayList<ArrayList<Resource> >();
    protected ArrayList<Resource> mResourcesLeft  = new ArrayList<Resource>();
    protected ArrayList<Resource> m_otherResources = new ArrayList<Resource>();
    protected Resource mLatestResource = null;
    protected ArrayList<Resource> mResourcesPickedUp = new ArrayList<Resource>();
    protected int mNStages;


    public ResourceManager(ArrayList<Resource> x, int n)
    {
        mAllResources =x;
        mNStages = n;
    }

    /**
     * Constructeur par defaut.
     * ATTENTION: doit être initialise par la fonction set(ArrayList<Resource> x, int n)
     */
    public ResourceManager()
    {
    }

    /**
     * Initialise les ressources
     * @param x La liste des ressources
     * @param n Le nombre d'étapes dans le jeu
     */
    public void set(ArrayList<Resource> x, int n)
    {
        mAllResources =x;
        mNStages = n;
    }

    /**
     * Ajout d'une ressource hors zone
     * @param r ressource hors zone
     */
    public void addOtherResource(Resource r)
    {
        m_otherResources.add(r);
    }

    /**
     * Récupere une ressource hors zone
     * @return
     */
    public ArrayList<Resource> getOtherResource()
    {
        return m_otherResources;
    }


    /**
     * Recupere une ressource de l'étape  courante qui nest pas encore trouvée
     * par son nom
     * @param name Le nom de la ressource
     * @return Null si inexistante sinon la ressource
     */
    public Resource getResourceLeftByName(String name)
    {
        for(int i=0; i<mResourcesLeft.size(); i++)
        {
            if(mResourcesLeft.get(i).getName().compareTo(name)==0)
                return mResourcesLeft.get(i);
        }

        for(int i=0; i<m_otherResources.size(); i++)
        {
            if(m_otherResources.get(i).getName().compareTo(name)==0)
                return m_otherResources.get(i);
        }
        return null;
    }

    public void clearPickedUpResources()
    {
        mResourcesPickedUp.clear();
    }

    public ArrayList<Resource> getResourceLeftByName(ArrayList<String> names)
    {
        ArrayList<Resource> res = new ArrayList<Resource>();
        for(int i=names.size()-1; i>=0; i--) {
            res.add(getResourceLeftByName(names.get(i)));
        }
        return res;
    }

    /**
     * Passe à l'étape N
     * @param stage Le numero de la prochaine etape
     */
    public void nextStage(int stage)
    {
        mResourcesAquiered.add(new ArrayList<Resource>());
        mResourcesLeft.clear();
        for(int i=0; i<mAllResources.size(); i++)
            if(mAllResources.get(i).getStage()==stage)
                mResourcesLeft.add(mAllResources.get(i));
    }

    /**
     * Recupere une ressource:
     *  -Ajout dans la liste des ressources (indifferent des etapes)
     *  -Ajout dans la liste d'inventaire (liste de liste par etape): mResourcesAquiered
     *  -Suppression de la liste des ressource à trouver (sauf si c'est une ressource hors zone)
     *
     * @param name
     * @return
     */
    public boolean pickUpResource(String name)
    {
        for(int i=0; i<mResourcesLeft.size(); i++)
        {
            if(mResourcesLeft.get(i).getName().compareTo(name)==0)
            {
                mResourcesPickedUp.add(mResourcesLeft.get(i));
                mResourcesAquiered.get(mResourcesAquiered.size()-1).add(mResourcesLeft.get(i));
                mLatestResource = mResourcesLeft.get(i);
                mResourcesLeft.remove(i);
                return true;
            }
        }

        for(int i=0; i<m_otherResources.size(); i++)
            if(m_otherResources.get(i).getName().compareTo(name)==0)
            {
                mResourcesPickedUp.add(m_otherResources.get(i));
                if(mResourcesAquiered.size()>0)mResourcesAquiered.get(mResourcesAquiered.size()-1).add(m_otherResources.get(i));
            }

        return false;
    }

    public ArrayList<Resource> getPickedUpResources()
    {
        return mResourcesPickedUp;
    }

    public ArrayList<Resource> getResourceLeft()
    {
        return mResourcesLeft;
    }

    public ArrayList<Resource> getResourceAquiered(int stage)
    {
        return mResourcesAquiered.get(stage-1);
    }


    public ArrayList< ArrayList<Resource> > getResourceAquiered()
    {
        return mResourcesAquiered;
    }

    /**
     * Renvoie le nombre de ressource acquise par le joueur depuis le debut de la partie
     * @return
     */
    public int getNResourceAquired()
    {
        int n=0;
        for(int i=0; i<mResourcesAquiered.size(); i++)
            n+=mResourcesAquiered.get(i).size();
        return n;
    }

    /**
     * Récupre toutes les ressources en un coups
     */
    public void pickAllResoucres()
    {
        mResourcesLeft.clear();
        mResourcesAquiered.clear();

        mResourcesAquiered.add(new ArrayList<Resource>());

        for(int i=0; i<m_otherResources.size(); i++)
            mResourcesAquiered.get(0).add(m_otherResources.get(i));

        for(int i=0; i<mNStages; i++)
        {
            ArrayList<Resource> ar = new ArrayList<Resource>();
            for(int j=0; j<mAllResources.size(); j++) {
                if (mAllResources.get(j).getStage() == i + 1)
                    ar.add(mAllResources.get(j));
            }
            mResourcesAquiered.add(ar);
        }


    }

    /**
     * Enleve le Modele 3D de la cache
     */
    public void deleteAll3DModel()
    {
        for(int i=0; i<mAllResources.size(); i++)
            mAllResources.get(i).delete3DModel();
    }

    /**
     * Le nombre de ressources acquise dans l'étape courante
     * @return
     */
    public int getNResourceAquiredByStage()
    {
        if(mResourcesAquiered.size()>0)
            return mResourcesAquiered.get(mResourcesAquiered.size()-1).size();
        return 0;
    }


    public int getNResourceLeft()
    {
        return mResourcesLeft.size();
    }

    public Resource getLatestResource()
    {
        return mLatestResource;
    }


    public boolean hasResource(String name)
    {
        for(int i=0; i<mResourcesAquiered.size(); i++)
        {
            for(int j=0; j<mResourcesAquiered.get(i).size(); j++)
                if(mResourcesAquiered.get(i).get(j).getName().compareTo(name)==0)
                    return true;
        }
        return false;
    }
}
