package app.mar.utils.game;

import android.app.Activity;

import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

import java.io.Serializable;
import java.util.ArrayList;

import app.mar.utils.geometry.GPSPoint;
import app.mar.utils.geometry.Shape;

/**
 * Created by ptitcois on 19/08/16.
 */
public class Area  implements Serializable {

    protected String            mName;
    protected ArrayList<String>            mResourceName = new ArrayList<String>();
    protected Shape             mShape = new Shape();
    protected ArrayList<Place>  mPlaces = new ArrayList<Place>();
    protected ArrayList<Resource>          mResource = new ArrayList<Resource>();

    public Area(JSONObject root, Activity act)
    {
        try {
            JSONArray ja = root.getJSONArray("coordinates");
            for(int i=0; i<ja.length(); i++) {
                JSONArray jaa = ja.getJSONArray(i);
                mShape.add(new GPSPoint(jaa.getDouble(0), jaa.getDouble(1)));
            }
            mName = root.getString("name");
            String res [] = root.getString("resource").split(";");
            for(int i=0; i<res.length; i++) mResourceName.add(res[i]);
            ja = root.getJSONArray("points");
            for(int i=0; i<ja.length(); i++) {
                mPlaces.add(new Place(ja.getJSONObject(i)));
            }
            for(int i=0; i<mResourceName.size(); i++)
                mResource.add(new Resource(mResourceName.get(i), act));
        } catch (JSONException e) {
            e.printStackTrace();
        }
    }

    public String getName() {
        return mName;
    }

    public ArrayList<String> getResourceNames() {
        return mResourceName;
    }

    public ArrayList<Resource> getResource() {
        return mResource;
    }
    public Resource getResource(int i) {
        return mResource.get(i);
    }
    public int countResource() {
        return mResource.size();
    }


    public int getStage()
    {
        if(mResource==null || mResource.size()<=0) return -1;
        int max=0;
        for(int i=0; i<mResource.size(); i++)
            if(mResource.get(i).getStage()>max)
                max=mResource.get(i).getStage();
        return max;
    }

    public Shape getShape() {
        return mShape;
    }

    public ArrayList<Place> getPlaces() {
        return mPlaces;
    }

    public int placesCount()
    {
        return mPlaces.size();
    }

    public Place getPlace(int i)
    {
        return mPlaces.get(i);
    }

    public JSONObject getJson()
    {
        JSONObject jo = new JSONObject();
        JSONArray ja = new JSONArray();

        try {
            for(int i=0; i<mPlaces.size(); i++)
                ja.put(mPlaces.get(i).getJson());
            jo.put("points", ja);
            jo.put("coordinates", mShape.getJson());
            jo.put("name", mName);
            jo.put("resource", mResourceName);
        }catch( Exception e)
        {
            e.printStackTrace();
        }
        return jo;
    }

    public boolean isOnArea(Player p)
    {
        GPSPoint player = p.getPosition();
        if(player == null)
            return false;

        return mShape.contains(player);

    }


    public boolean isOnPlace(Player p)
    {
        for(int i=0; i<mPlaces.size(); i++)
            if(mPlaces.get(i).isPlayerOn(p))
                return true;
        return false;
    }

    public boolean isOnDirection(Player p)
    {

        for(int i=0; i<mPlaces.size(); i++)
            if(mPlaces.get(i).isOnDirection(p))
                return true;
        return false;
    }

    public double getDistanceToNextPlace(Player p)
    {
        double min = 32*1000*1000*1000;
        for(int i=0; i<mPlaces.size(); i++)
            if(mPlaces.get(i).getDistance(p)<min)
                min=mPlaces.get(i).getDistance(p);
        return min;
    }

    public String toString(Player p)
    {
        String str = "";
        str+="["+mName+" "+mPlaces.get(0).getDistance(p)+"m ]";
        return str;
    }
}
